@@ -30,6 +30,7 @@
 
 static unsigned int debug_mask;
 static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, char **error_string);
+static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, char **error_string);
 
 static char *nodename_env;
 static int expected_votes;
@@ -53,7 +54,8 @@ static unsigned int cluster_parent_handle;
  * Exports the interface for the service
  */
 static struct config_iface_ver0 cmanpreconfig_iface_ver0 = {
-	.config_readconfig        = cmanpre_readconfig
+	.config_readconfig        = cmanpre_readconfig,
+	.config_reloadconfig      = cmanpre_reloadconfig
 };
 
 static struct lcr_iface ifaces_ver0[2] = {
@@ -806,6 +808,17 @@ static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
 				 "corosync_cman", strlen("corosync_cman") + 1);
 	objdb->object_key_create(object_handle, "ver", strlen("ver"),
 				 "0", 2);
+
+	/* Define cman as the quorum provider for corosync */
+	objdb->object_find_create(OBJECT_PARENT_HANDLE, "quorum", strlen("quorum"), &find_handle);
+	if (objdb->object_find_next(find_handle, &object_handle) != 0) {
+		objdb->object_create(OBJECT_PARENT_HANDLE, &object_handle,
+					"quorum", strlen("quorum"));
+	}
+	objdb->object_find_destroy(find_handle);
+
+	objdb->object_key_create(object_handle, "provider", strlen("provider"),
+				 "quorum_cman", strlen("quorum_cman") + 1);
 }
 
 /* If ccs is not available then use some defaults */
@@ -1011,6 +1024,69 @@ static int get_cman_globals(struct objdb_iface_ver0 *objdb)
 	return 0;
 }
 
+static int cmanpre_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, char **error_string)
+{
+	int ret = -1;
+	unsigned int object_handle;
+	unsigned int find_handle;
+	unsigned int cluster_parent_handle_new;
+
+	/* don't reload if we've been told to run configless */
+	if (getenv("CMAN_NOCONFIG")) {
+		sprintf(error_reason, "Config not updated because we were run with cman_tool -X");
+		ret = 0;
+		goto err;
+	}
+
+	/* find both /cluster entries */
+	objdb->object_find_create(OBJECT_PARENT_HANDLE, "cluster", strlen("cluster"), &find_handle);
+	objdb->object_find_next(find_handle, &cluster_parent_handle);
+	if (!cluster_parent_handle) {
+		sprintf (error_reason, "%s", "Cannot find old /cluster/ key in configuration\n");
+		goto err;
+	}
+	objdb->object_find_next(find_handle, &cluster_parent_handle_new);
+	if (!cluster_parent_handle_new) {
+		sprintf (error_reason, "%s", "Cannot find new /cluster/ key in configuration\n");
+		goto err;
+	}
+	objdb->object_find_destroy(find_handle);
+
+	/* destroy the old one */
+	objdb->object_destroy(cluster_parent_handle);
+
+	/* update the reference to the new config */
+	cluster_parent_handle = cluster_parent_handle_new;
+
+	/* destroy top level /logging */
+	objdb->object_find_create(OBJECT_PARENT_HANDLE, "logging", strlen("logging"), &find_handle);
+	objdb->object_find_next(find_handle, &object_handle);
+	objdb->object_find_destroy(find_handle);
+	if (object_handle) {
+		objdb->object_destroy(object_handle);
+	}
+
+	/* copy /cluster/logging to /logging */
+	ret = copy_tree_to_root(objdb, "logging", 0);
+
+	/* destroy top level /totem */
+	objdb->object_find_create(OBJECT_PARENT_HANDLE, "totem", strlen("totem"), &find_handle);
+	objdb->object_find_next(find_handle, &object_handle);
+	objdb->object_find_destroy(find_handle);
+	if (object_handle) {
+		objdb->object_destroy(object_handle);
+	}
+
+	/* copy /cluster/totem to /totem */
+	ret = copy_tree_to_root(objdb, "totem", 0);
+
+	return 0;
+
+err:
+	*error_string = error_reason;
+	return ret;
+}
+
 static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
 {
 	int ret = 0;
@@ -1024,21 +1100,21 @@ static int cmanpre_readconfig(struct objdb_iface_ver0 *objdb, char **error_strin
         objdb->object_find_next(find_handle, &cluster_parent_handle);
 	objdb->object_find_destroy(find_handle);
 	if (!cluster_parent_handle) {
-		sprintf (error_reason, "%s", "Cannot find /cluster/ key in configuration\n");
-		return -1;
+                objdb->object_create(OBJECT_PARENT_HANDLE, &cluster_parent_handle,
+				     "cluster", strlen("cluster"));
+	}
+	else {
+		/* Move these to a place where corosync expects to find them */
+		ret = copy_tree_to_root(objdb, "totem", 0);
+		ret = copy_tree_to_root(objdb, "logging", 0);
+		ret = copy_tree_to_root(objdb, "event", 0);
+		ret = copy_tree_to_root(objdb, "amf", 0);
+		ret = copy_tree_to_root(objdb, "aisexec", 0);
+		ret = copy_tree_to_root(objdb, "service", 1);
 	}
-
-	/* Move these to a place where corosync expects to find them */
-	ret = copy_tree_to_root(objdb, "totem", 0);
-	ret = copy_tree_to_root(objdb, "logging", 0);
-	ret = copy_tree_to_root(objdb, "event", 0);
-	ret = copy_tree_to_root(objdb, "amf", 0);
-	ret = copy_tree_to_root(objdb, "aisexec", 0);
-	ret = copy_tree_to_root(objdb, "service", 1);
 
 	objdb->object_find_create(cluster_parent_handle, "cman", strlen("cman"), &find_handle);
 	if (objdb->object_find_next(find_handle, &object_handle)) {
-
                 objdb->object_create(cluster_parent_handle, &object_handle,
 					"cman", strlen("cman"));
         }
