@@ -22,7 +22,7 @@ char *_ccs_get_xpathlite(confdb_handle_t handle, unsigned int connection_handle,
 char *_ccs_get_fullxpath(confdb_handle_t handle, unsigned int connection_handle,
 			 const char *query, int list)
     __attribute__ ((visibility("hidden")));
-int xpathfull_init(confdb_handle_t handle, int ccs_handle)
+int xpathfull_init(confdb_handle_t handle)
     __attribute__ ((visibility("hidden")));
 void xpathfull_finish() __attribute__ ((visibility("hidden")));
 
