@@ -720,7 +720,8 @@ static int check_uncontrolled_entries(void)
 
 void cluster_dead(int ci)
 {
-	log_error("cluster is down, exiting");
+	if (!cluster_down)
+		log_error("cluster is down, exiting");
 	daemon_quit = 1;
 	cluster_down = 1;
 }
@@ -768,9 +769,21 @@ static void loop(void)
 			goto out;
 		client_add(rv, process_groupd, cluster_dead);
 
-		group_mode = GROUP_LIBGROUP;
-		if (cfgd_groupd_compat == 2)
-			set_group_mode();
+		switch (cfgd_groupd_compat) {
+		case 1:
+			group_mode = GROUP_LIBGROUP;
+			rv = 0;
+			break;
+		case 2:
+			rv = set_group_mode();
+			break;
+		default:
+			log_error("inval groupd_compat %d", cfgd_groupd_compat);
+			rv = -1;
+			break;
+		}
+		if (rv < 0)
+			goto out;
 	}
 	log_debug("group_mode %d compat %d", group_mode, cfgd_groupd_compat);
 
@@ -874,8 +887,8 @@ static void print_usage(void)
 	printf("\n");
 	printf("Options:\n");
 	printf("\n");
-	printf("  -D           Enable debugging code and don't fork\n");
-	printf("  -L <num>     Enable (1) or disable (0) debugging to logsys (default %d)\n", DEFAULT_DEBUG_LOGSYS);
+	printf("  -D           Enable debugging to stderr and don't fork\n");
+	printf("  -L           Enable debugging to log file\n");
 	printf("  -g <num>     groupd compatibility mode, 0 off, 1 on, 2 detect (default %d)\n", DEFAULT_GROUPD_COMPAT);
 	printf("               0: use libcpg, no backward compat, best performance\n");
 	printf("               1: use libgroup for compat with cluster2/rhel5\n");
@@ -899,7 +912,7 @@ static void print_usage(void)
 	printf("\n");
 }
 
-#define OPTION_STRING	"L:g:cj:f:Dn:O:hVSse:r:"
+#define OPTION_STRING	"Lg:cj:f:Dn:O:hVSse:r:"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -916,8 +929,8 @@ static void read_arguments(int argc, char **argv)
 			break;
 
 		case 'L':
-			optd_debug_logsys = 1;
-			cfgd_debug_logsys = atoi(optarg);
+			optd_debug_logfile = 1;
+			cfgd_debug_logfile = 1;
 			break;
 
 		case 'g':
@@ -989,9 +1002,9 @@ static void read_arguments(int argc, char **argv)
 		};
 	}
 
-	if (!optd_debug_logsys && getenv("FENCED_DEBUG")) {
-		optd_debug_logsys = 1;
-		cfgd_debug_logsys = atoi(getenv("FENCED_DEBUG"));
+	if (getenv("FENCED_DEBUG")) {
+		optd_debug_logfile = 1;
+		cfgd_debug_logfile = 1;
 	}
 }
 
@@ -1023,6 +1036,7 @@ int main(int argc, char **argv)
 		umask(0);
 	}
 	init_logging();
+	log_level(LOG_INFO, "fenced %s", RELEASE_VERSION);
 	signal(SIGTERM, sigterm_handler);
 	set_oom_adj(-16);
 
