@@ -7,7 +7,6 @@
 #include <string.h>
 #include <errno.h>
 #include <time.h>
-#include <syslog.h>
 
 #include "ccs.h"
 
@@ -15,11 +14,11 @@
 #define MAX_DEVICES		8
 #define MAX_AGENT_ARGS_LEN	512
 
-#define METHOD_NAME_PATH        "/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[%d]/@name"
-#define DEVICE_NAME_PATH        "/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[@name=\"%s\"]/device[%d]/@name"
-#define NODE_FENCE_ARGS_PATH    "/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[@name=\"%s\"]/device[%d]/@*"
-#define AGENT_NAME_PATH         "/cluster/fencedevices/fencedevice[@name=\"%s\"]/@agent"
-#define FENCE_DEVICE_ARGS_PATH  "/cluster/fencedevices/fencedevice[@name=\"%s\"]/@*"
+#define METHOD_NAME_PATH		"/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[%d]/@name"
+#define DEVICE_NAME_PATH		"/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[@name=\"%s\"]/device[%d]/@name"
+#define NODE_FENCE_ARGS_PATH	"/cluster/clusternodes/clusternode[@name=\"%s\"]/fence/method[@name=\"%s\"]/device[%d]/@*"
+#define AGENT_NAME_PATH			"/cluster/fencedevices/fencedevice[@name=\"%s\"]/@agent"
+#define FENCE_DEVICE_ARGS_PATH	"/cluster/fencedevices/fencedevice[@name=\"%s\"]/@*"
 
 
 
@@ -36,7 +35,9 @@ static void display_agent_output(const char *agent, int fd)
 			break;
 		} else if (ret > 0) {
 			buf[ret] = '\0';
+			/*
 			syslog(LOG_ERR, "agent \"%s\" reports: %s", agent, buf);
+			*/
 		}
 	} while (ret > 0);
 }
@@ -57,17 +58,17 @@ static int run_agent(char *agent, char *args)
 
 	if (pipe(fd1))
 		goto fail;
-  	pr_fd = fd1[0];
-  	cw_fd = fd1[1];
+	pr_fd = fd1[0];
+	cw_fd = fd1[1];
 
-  	if (pipe(fd2))
-   		goto fail;
-  	cr_fd = fd2[0];
-  	pw_fd = fd2[1];
+	if (pipe(fd2))
+		goto fail;
+	cr_fd = fd2[0];
+	pw_fd = fd2[1];
 
 	pid = fork();
 	if (pid < 0)
-   		goto fail;
+		goto fail;
 
 	if (pid) {
 		/* parent */
@@ -125,10 +126,10 @@ static int run_agent(char *agent, char *args)
 }
 
 static int make_args(int cd, char *victim, char *method, int d,
-		     char *device, char **args_out)
+				char *device, char **args_out)
 {
 	char path[256], *args, *str;
-	int error;
+	int error, cnt = 0;
 
 	args = malloc(MAX_AGENT_ARGS_LEN);
 	if (!args)
@@ -144,6 +145,7 @@ static int make_args(int cd, char *victim, char *method, int d,
 		error = ccs_get_list(cd, path, &str);
 		if (error || !str)
 			break;
+		++cnt;
 
 		if (!strncmp(str, "name=", 5)) {
 			free(str);
@@ -164,6 +166,7 @@ static int make_args(int cd, char *victim, char *method, int d,
 		error = ccs_get_list(cd, path, &str);
 		if (error || !str)
 			break;
+		++cnt;
 
 		if (!strncmp(str, "name=", 5)) {
 			free(str);
@@ -175,6 +178,8 @@ static int make_args(int cd, char *victim, char *method, int d,
 		free(str);
 	}
 
+	if (cnt)
+		error = 0;
 	if (error) {
 		free(args);
 		args = NULL;
@@ -249,7 +254,7 @@ static int count_devices(int cd, char *victim, char *method)
 }
 
 static int use_device(int cd, char *victim, char *method, int d,
-		      char *device)
+				char *device)
 {
 	char path[256], *agent, *args = NULL;
 	int error;
@@ -280,11 +285,9 @@ int fence_node(char *victim)
 	char *victim_nodename = NULL;
 	int num_methods, num_devices, m, d, error = -1, cd;
 
-	cd = ccs_force_connect(NULL, 0);
-	if (cd < 0) {
-		syslog(LOG_ERR, "cannot connect to ccs %d\n", cd);
+	cd = ccs_connect();
+	if (cd < 0)
 		return -1;
-	}
 
 	if (ccs_lookup_nodename(cd, victim, &victim_nodename) == 0)
 		victim = victim_nodename;
@@ -294,23 +297,8 @@ int fence_node(char *victim)
 	for (m = 0; m < num_methods; m++) {
 
 		error = get_method(cd, victim, m, &method);
-
-		/* if the connection timed out while we were trying 
-		 * to fence, try to open the connection again
-		 */
-		if (error == -EBADR) {
-			syslog(LOG_INFO, "ccs connection timed out, "
-				"retrying\n");
-
-			while ((cd = ccs_connect()) < 0)
-				sleep(1);
-			
-			error = get_method(cd, victim, m, &method);
-
 		if (error)
 			continue;
-		} else if (error)
-			continue;
 
 		/* if num_devices is zero we should return an error */
 		error = -1;
@@ -344,4 +332,3 @@ int fence_node(char *victim)
 
 	return error;
 }
-
