@@ -1,16 +1,3 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
-**
-**  This copyrighted material is made available to anyone wishing to use,
-**  modify, copy, or redistribute it subject to the terms and conditions
-**  of the GNU General Public License v.2.
-**
-*******************************************************************************
-******************************************************************************/
-
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -87,15 +74,19 @@ int set_dotdot_dir(struct gfs2_sbd *sbp, uint64_t childblock,
 }
 
 static int check_eattr_indir(struct gfs2_inode *ip, uint64_t block,
-			    uint64_t parent, struct gfs2_buffer_head **bh, void *private)
+			    uint64_t parent, struct gfs2_buffer_head **bh,
+			     enum update_flags *want_updated, void *private)
 {
-	*bh = bread(ip->i_sbd, block);
+	*want_updated = not_updated;
+	*bh = bread(&ip->i_sbd->buf_list, block);
 	return 0;
 }
 static int check_eattr_leaf(struct gfs2_inode *ip, uint64_t block,
-			    uint64_t parent, struct gfs2_buffer_head **bh, void *private)
+			    uint64_t parent, struct gfs2_buffer_head **bh,
+			    enum update_flags *want_updated, void *private)
 {
-	*bh = bread(ip->i_sbd, block);
+	*want_updated = not_updated;
+	*bh = bread(&ip->i_sbd->buf_list, block);
 	return 0;
 }
 
@@ -154,7 +145,7 @@ static int check_file_type(uint8_t de_type, uint8_t block_type)
 int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 		 struct gfs2_dirent *prev_de,
 		 struct gfs2_buffer_head *bh, char *filename,
-		 int *update, uint16_t *count, void *priv)
+		 enum update_flags *update, uint16_t *count, void *priv)
 {
 	struct gfs2_sbd *sbp = ip->i_sbd;
 	struct gfs2_block_query q = {0};
@@ -167,6 +158,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 	struct gfs2_dirent dentry, *de;
 	uint32_t calculated_hash;
 
+	*update = not_updated;
 	memset(&dentry, 0, sizeof(struct gfs2_dirent));
 	gfs2_dirent_in(&dentry, (char *)dent);
 	de = &dentry;
@@ -185,7 +177,8 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			"\tName length = %u\n",
 			de->de_rec_len,
 			de->de_name_len);
-		gfs2_block_set(bl, ip->i_di.di_num.no_addr, gfs2_meta_inval);
+		gfs2_block_set(sbp, bl, ip->i_di.di_num.no_addr,
+			       gfs2_meta_inval);
 		return 1;
 		/* FIXME: should probably delete the entry here at the
 		 * very least - maybe look at attempting to fix it */
@@ -226,7 +219,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				 "Clear directory entry tp out of range block? (y/n) ")) {
 			log_err("Clearing %s\n", tmp_name);
 			dirent2_del(ip, bh, prev_de, dent);
-			*update = 1;
+			*update = updated;
 			return 1;
 		} else {
 			log_err("Directory entry to out of range block remains\n");
@@ -235,7 +228,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			return 0;
 		}
 	}
-	if(gfs2_block_check(bl, de->de_inum.no_addr, &q)) {
+	if(gfs2_block_check(sbp, bl, de->de_inum.no_addr, &q)) {
 		stack;
 		return -1;
 	}
@@ -251,7 +244,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 		if(query(&opts, "Clear entry to inode containing bad blocks? (y/n)")) {
 
 			entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
-			check_inode_eattr(entry_ip, &clear_eattrs);
+			check_inode_eattr(entry_ip, update, &clear_eattrs);
 			fsck_inode_put(entry_ip, not_updated);
 
 			/* FIXME: make sure all blocks referenced by
@@ -259,8 +252,9 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 
 			dirent2_del(ip, bh, prev_de, dent);
 
-			gfs2_block_set(bl, de->de_inum.no_addr, gfs2_meta_inval);
-			*update = 1;
+			gfs2_block_set(sbp, bl, de->de_inum.no_addr,
+				       gfs2_meta_inval);
+			*update = updated;
 			return 1;
 		} else {
 			log_warn("Entry to inode containing bad blocks remains\n");
@@ -286,7 +280,7 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			 * this inode are cleared in the bitmap */
 
 			dirent2_del(ip, bh, prev_de, dent);
-			*update = 1;
+			*update = updated;
 			log_warn("Directory entry '%s' cleared\n", tmp_name);
 			return 1;
 		} else {
@@ -310,11 +304,11 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 				 block_type_string(&q));
 		if(query(&opts, "Clear stale directory entry? (y/n) ")) {
 			entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
-			check_inode_eattr(entry_ip, &clear_eattrs);
+			check_inode_eattr(entry_ip, update, &clear_eattrs);
 			fsck_inode_put(entry_ip, not_updated);
 
 			dirent2_del(ip, bh, prev_de, dent);
-			*update  = 1;
+			*update = updated;
 			return 1;
 		} else {
 			log_err("Stale directory entry remains\n");
@@ -334,11 +328,12 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			if(query(&opts, "Clear duplicate '.' entry? (y/n) ")) {
 
 				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
-				check_inode_eattr(entry_ip, &clear_eattrs);
+				check_inode_eattr(entry_ip, update,
+						  &clear_eattrs);
 				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
-				*update  = 1;
+				*update = updated;
 				return 1;
 			} else {
 				log_err("Duplicate '.' entry remains\n");
@@ -365,11 +360,12 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			if(query(&opts, "Remove '.' reference? (y/n) ")) {
 				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
-				check_inode_eattr(entry_ip, &clear_eattrs);
+				check_inode_eattr(entry_ip, update,
+						  &clear_eattrs);
 				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
-				*update = 1;
+				*update = updated;
 				return 1;
 
 			} else {
@@ -385,7 +381,6 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 
 		ds->dotdir = 1;
 		increment_link(sbp, de->de_inum.no_addr);
-		*update = (opts.no ? not_updated : updated);
 		(*count)++;
 		ds->entry_count++;
 
@@ -400,7 +395,8 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 			if(query(&opts, "Clear duplicate '..' entry? (y/n) ")) {
 
 				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
-				check_inode_eattr(entry_ip, &clear_eattrs);
+				check_inode_eattr(entry_ip, update,
+						  &clear_eattrs);
 				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
@@ -425,7 +421,8 @@ int check_dentry(struct gfs2_inode *ip, struct gfs2_dirent *dent,
 					ip->i_di.di_num.no_addr, ip->i_di.di_num.no_addr);
 			if(query(&opts, "Clear bad '..' directory entry? (y/n) ")) {
 				entry_ip = fsck_load_inode(sbp, de->de_inum.no_addr);
-				check_inode_eattr(entry_ip, &clear_eattrs);
+				check_inode_eattr(entry_ip, update,
+						  &clear_eattrs);
 				fsck_inode_put(entry_ip, not_updated);
 
 				dirent2_del(ip, bh, prev_de, dent);
@@ -524,13 +521,14 @@ int check_system_dir(struct gfs2_inode *sysinode, const char *dirname,
 	char *filename;
 	int filename_len;
 	char tmp_name[256];
-	int update=0, error = 0;
+	enum update_flags update = not_updated;
+	int error = 0;
 
 	log_info("Checking system directory inode '%s'\n", dirname);
 
 	if (sysinode) {
 		iblock = sysinode->i_di.di_num.no_addr;
-		if(gfs2_block_check(bl, iblock, &ds.q)) {
+		if(gfs2_block_check(sysinode->i_sbd, bl, iblock, &ds.q)) {
 			iblock = sysinode->i_di.di_num.no_addr;
 		}
 	}
@@ -548,10 +546,10 @@ int check_system_dir(struct gfs2_inode *sysinode, const char *dirname,
 		return -1;
 	}
 	if (error > 0)
-		gfs2_block_set(bl, iblock, gfs2_meta_inval);
+		gfs2_block_set(sysinode->i_sbd, bl, iblock, gfs2_meta_inval);
 
 	bh = bhold(sysinode->i_bh);
-	if(check_inode_eattr(sysinode, &pass2_fxns)) {
+	if(check_inode_eattr(sysinode, &update, &pass2_fxns)) {
 		stack;
 		return -1;
 	}
@@ -668,7 +666,7 @@ int pass2(struct gfs2_sbd *sbp)
 		if (is_system_dir(sbp, i))
 			continue;
 
-		if(gfs2_block_check(bl, i, &q)) {
+		if(gfs2_block_check(sbp, bl, i, &q)) {
 			log_err("Can't get block %"PRIu64 " (0x%" PRIx64
 					") from block list\n", i, i);
 			return -1;
@@ -727,9 +725,9 @@ int pass2(struct gfs2_sbd *sbp)
 				} else
 					log_err("Directory entry to invalid inode remains.\n");
 			}
-			gfs2_block_set(bl, i, gfs2_meta_inval);
+			gfs2_block_set(sbp, bl, i, gfs2_meta_inval);
 		}
-		bh = bread(sbp, i);
+		bh = bread(&sbp->buf_list, i);
 		ip = fsck_inode_get(sbp, bh);
 		if(!ds.dotdir) {
 			log_err("No '.' entry found\n");
@@ -755,7 +753,7 @@ int pass2(struct gfs2_sbd *sbp)
 		}
 		fsck_inode_put(ip, not_updated); /* does a brelse */
 
-		bh = bread(sbp, i);
+		bh = bread(&sbp->buf_list, i);
 		ip = fsck_inode_get(sbp, bh);
 		if(ip->i_di.di_entries != ds.entry_count) {
 			log_err("Entries is %d - should be %d for inode block %" PRIu64
