@@ -182,7 +182,23 @@ static void set_group_mode(void)
 
 		group_mode = node->ver.group_mode;
 
-		log_print("group mode is %s", mode_str(group_mode));
+		switch (group_mode) {
+		case GROUP_PENDING:
+			/* shouldn't happen */
+			log_level(LOG_INFO, "groupd compatibility mode 2 ver");
+			break;
+		case GROUP_LIBGROUP:
+			log_level(LOG_INFO, "groupd compatibility mode 1 ver");
+			break;
+		case GROUP_LIBCPG:
+			log_level(LOG_INFO, "groupd compatibility mode 0 ver");
+			break;
+		default:
+			log_level(LOG_INFO, "groupd compatibility mode %d ver",
+				  group_mode);
+			break;
+		}
+
 		log_debug("set_group_mode %s matching nodeid %d got_from %d",
 			  mode_str(group_mode), node->nodeid, node->got_from);
 		break;
@@ -484,6 +500,9 @@ void process_groupd_confchg(void)
 	memcpy(&groupd_cpg_member, &saved_member, sizeof(saved_member));
 	groupd_cpg_member_count = saved_member_count;
 
+	if (group_mode != GROUP_LIBGROUP)
+		return;
+
 	for (i = 0; i < saved_member_count; i++) {
 		if (saved_member[i].nodeid == our_nodeid &&
 		    saved_member[i].pid == (uint32_t) getpid()) {
