@@ -1095,6 +1095,14 @@ static void prepare_plocks(struct lockspace *ls)
 
 	set_plock_ckpt_node(ls);
 
+	/* there is no node with plock state, so there's no syncing to do */
+
+	if (!ls->plock_ckpt_node) {
+		ls->need_plocks = 0;
+		ls->save_plocks = 0;
+		return;
+	}
+
 	/* We save all plock messages from the time that the low node saves
 	   existing plock state in the ckpt to the time that we read that state
 	   from the ckpt. */
@@ -1320,16 +1328,12 @@ static void confchg_cb(cpg_handle_t handle, struct cpg_name *group_name,
 	list_for_each_entry(memb, &cg->removed, list)
 		purge_plocks(ls, memb->nodeid, 0);
 
-#if 0
-	/* deadlock code needs to adjust per a confchg, is this the right
-	   way/place for this? */
+	apply_changes(ls);
 
 	deadlk_confchg(ls, member_list, member_list_entries,
 		       left_list, left_list_entries,
 		       joined_list, joined_list_entries);
-#endif
 
-	apply_changes(ls);
 }
 
 static void dlm_header_in(struct dlm_header *hd)
@@ -1486,11 +1490,20 @@ int dlm_join_lockspace(struct lockspace *ls)
 	cpg_error_t error;
 	cpg_handle_t h;
 	struct cpg_name name;
-	int i = 0, fd, ci;
+	int i = 0, fd, ci, rv;
+	int unused;
+
+	rv = fence_in_progress(&unused);
+	if (cfgd_enable_fencing && rv < 0) {
+		log_error("dlm_join_lockspace no fence domain");
+		rv = -1;
+		goto fail_free;
+	}
 
 	error = cpg_initialize(&h, &cpg_callbacks);
 	if (error != CPG_OK) {
 		log_error("cpg_initialize error %d", error);
+		rv = -1;
 		goto fail_free;
 	}
 
@@ -1525,6 +1538,7 @@ int dlm_join_lockspace(struct lockspace *ls)
 	if (error != CPG_OK) {
 		log_error("cpg_join error %d", error);
 		cpg_finalize(h);
+		rv = -1;
 		goto fail;
 	}
 
@@ -1535,8 +1549,9 @@ int dlm_join_lockspace(struct lockspace *ls)
 	client_dead(ci);
 	cpg_finalize(h);
  fail_free:
+	set_sysfs_event_done(ls->name, rv);
 	free_ls(ls);
-	return error;
+	return rv;
 }
 
 /* received an "offline" uevent from dlm-kernel */
