@@ -32,8 +32,9 @@
 #include <dirent.h>
 #include <openais/saAis.h>
 #include <openais/saCkpt.h>
+
 #include <corosync/cpg.h>
-#include <corosync/engine/logsys.h>
+#include <liblogthread.h>
 
 #include <linux/dlmconstants.h>
 #include "libdlmcontrol.h"
@@ -46,7 +47,7 @@
    The libcpg limit is larger at CPG_MAX_NAME_LENGTH 128.  Our cpg name includes
    a "dlm:" prefix before the lockspace name. */
 
-/* Maximum members of a lockspace, should match CPG_MEMBERS_MAX in corosync/cpg.h.
+/* Maximum members of a ls, should match CPG_MEMBERS_MAX in corosync/cpg.h.
    There are no max defines in dlm-kernel for lockspace members. */
 
 #define MAX_NODES	128
@@ -91,31 +92,26 @@ extern uint32_t old_plock_minor;
 
 void daemon_dump_save(void);
 
-#define log_debug(fmt, args...) \
+#define log_level(lvl, fmt, args...) \
 do { \
 	snprintf(daemon_debug_buf, 255, "%ld " fmt "\n", time(NULL), ##args); \
 	daemon_dump_save(); \
+	logt_print(lvl, fmt "\n", ##args); \
 	if (daemon_debug_opt) \
 		fprintf(stderr, "%s", daemon_debug_buf); \
-	if (cfgd_debug_logsys) \
-		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
 } while (0)
 
+#define log_debug(fmt, args...) log_level(LOG_DEBUG, fmt, ##args)
+#define log_error(fmt, args...) log_level(LOG_ERR, fmt, ##args)
+
 #define log_group(ls, fmt, args...) \
 do { \
 	snprintf(daemon_debug_buf, 255, "%ld %s " fmt "\n", time(NULL), \
 		 (ls)->name, ##args); \
 	daemon_dump_save(); \
+	logt_print(LOG_DEBUG, fmt "\n", ##args); \
 	if (daemon_debug_opt) \
 		fprintf(stderr, "%s", daemon_debug_buf); \
-	if (cfgd_debug_logsys) \
-		log_printf(LOG_DEBUG, "%s", daemon_debug_buf); \
-} while (0)
-
-#define log_error(fmt, args...) \
-do { \
-	log_debug(fmt, ##args); \
-	log_printf(LOG_ERR, fmt, ##args); \
 } while (0)
 
 #define log_plock(ls, fmt, args...) \
@@ -231,7 +227,7 @@ int setup_ccs(void);
 void close_ccs(void);
 void read_ccs_name(char *path, char *name);
 void read_ccs_yesno(char *path, int *yes, int *no);
-void read_ccs_int(char *path, int *config_val);
+int read_ccs_int(char *path, int *config_val);
 int get_weight(int nodeid, char *lockspace);
 
 /* cpg.c */
@@ -260,6 +256,11 @@ void receive_checkpoint_ready(struct lockspace *ls, struct dlm_header *hd,
 void receive_cycle_start(struct lockspace *ls, struct dlm_header *hd, int len);
 void receive_cycle_end(struct lockspace *ls, struct dlm_header *hd, int len);
 void receive_cancel_lock(struct lockspace *ls, struct dlm_header *hd, int len);
+void deadlk_confchg(struct lockspace *ls,
+	struct cpg_address *member_list, int member_list_entries,
+	struct cpg_address *left_list, int left_list_entries,
+	struct cpg_address *joined_list, int joined_list_entries);
+
 
 /* main.c */
 int do_read(int fd, void *buf, size_t count);
@@ -320,7 +321,7 @@ int set_lockspaces_group(int *count,
 	struct dlmc_lockspace **lss_out);
 int set_lockspace_nodes_group(struct lockspace *ls, int option,
 	int *node_count, struct dlmc_node **nodes);
-void set_group_mode(void);
+int set_group_mode(void);
 #else
 static inline int setup_groupd(void) { return -1; }
 static inline void close_groupd(void) { }
@@ -335,7 +336,7 @@ static inline int set_lockspaces_group(int *count,
 	struct dlmc_lockspace **lss_out) { return -1; }
 static inline int set_lockspace_nodes_group(struct lockspace *ls, int option,
 	int *node_count, struct dlmc_node **nodes) { return -1; }
-static inline void set_group_mode(void) { }
+int inline void set_group_mode(void) { return 0; }
 #endif
 
 /* logging.c */
