@@ -1,21 +1,3 @@
-/*
-  Copyright Red Hat, Inc. 2004
-
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2, or (at your option) any
-  later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING.  If not, write to the
-  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-  MA 02139, USA.
-*/
 #ifndef _RESLIST_H
 #define _RESLIST_H
 
@@ -45,6 +27,7 @@
 #define RF_DESTROY	(1<<8)	/** Resource rule flag: Destroy this
 				  resource class if you delete it from
 				  the configuration */
+#define RF_ENFORCE_TIMEOUTS (1<<9) /** Enforce timeouts for this node */
 
 
 
@@ -202,7 +185,7 @@ void deconstruct_domains(fod_t **domains);
 void print_domains(fod_t **domains);
 int node_should_start(int nodeid, cluster_member_list_t *membership,
 		      char *rg_name, fod_t **domains);
-int node_domain_set(fod_t *domain, int **ret, int *retlen);
+int node_domain_set(fod_t **domains, char *name, int **ret, int *retlen, int *flags);
 int node_domain_set_safe(char *domainname, int **ret, int *retlen, int *flags);
 
 
