@@ -510,6 +510,7 @@ cluster_msg_receive(msgctx_t *ctx, void *msg, size_t maxlen, int timeout)
 	case M_PORTOPENED:
 	case M_PORTCLOSED:
 	case M_TRY_SHUTDOWN:
+	case M_CONFIG_UPDATE:
 		pthread_mutex_lock(&ctx->u.cluster_info.mutex);
 
 		n = ctx->u.cluster_info.queue;
@@ -709,7 +710,7 @@ queue_for_context(msgctx_t *ctx, char *buf, int len)
 	msg_q_t *node;
 
 	if (ctx->type != MSG_CLUSTER) {
-		log_printf(LOG_WARNING, "%s called on invalid context %p\n",
+		logt_print(LOG_WARNING, "%s called on invalid context %p\n",
 		       __FUNCTION__, ctx);
 		return;
 	}
@@ -828,7 +829,7 @@ process_cman_msg(cman_handle_t h, void *priv, char *buf, int len,
 		    contexts[m->dest_ctx]->type != MSG_CLUSTER) {
 			/* XXX Work around bug where M_CLOSE is called
 			   on a context which has been destroyed */
-			log_printf(LOG_WARNING, "Ignoring M_CLOSE for destroyed "
+			logt_print(LOG_WARNING, "Ignoring M_CLOSE for destroyed "
 			       "context %d\n", m->dest_ctx);
 		} else {
 			queue_for_context(contexts[m->dest_ctx], buf, len);
@@ -993,14 +994,24 @@ process_cman_event(cman_handle_t handle, void *private, int reason, int arg)
 	memset(msg, 0, sizeof(int)*2 +sizeof(cluster_msg_hdr_t));
 
 	switch(reason) {
-#if defined(LIBCMAN_VERSION) && LIBCMAN_VERSION >= 2
+#if defined(LIBCMAN_VERSION)
+#if LIBCMAN_VERSION >= 2
 	case CMAN_REASON_PORTOPENED:
 		msg->msg_control = M_PORTOPENED;
 		break;
 	case CMAN_REASON_TRY_SHUTDOWN:
 		msg->msg_control = M_TRY_SHUTDOWN;
 		break;
+#if LIBCMAN_VERSION >= 3
+	case CMAN_REASON_CONFIG_UPDATE:
+		msg->msg_control = M_CONFIG_UPDATE;
+#ifdef DEBUG
+		printf("M_CONFIG_UPDATE received!\n");
 #endif
+		break;
+#endif /* >= 3 */
+#endif /* >= 2 */
+#endif /* defined... */
 	case CMAN_REASON_PORTCLOSED:
 		msg->msg_control = M_PORTCLOSED;
 		break;
