@@ -32,13 +32,11 @@ void set_transition_throttling(int);
 void node_event(int, int, int, int);
 void node_event_q(int, int, int, int);
 int daemon_init(char *);
-int init_resource_groups(int, int);
 void kill_resource_groups(void);
 void set_my_id(int);
 void flag_shutdown(int sig);
 void hard_exit(void);
 int send_rg_states(msgctx_t *, int);
-int check_config_update(int *, int *);
 int svc_exists(char *);
 int watchdog_init(void);
 int32_t master_event_callback(char *key, uint64_t viewno, void *data, uint32_t datalen);
@@ -138,25 +136,25 @@ membership_update(void)
 		if (!rg_quorate())
 			return -1;
 
-		log_printf(LOG_EMERG, "#1: Quorum Dissolved\n");
+		logt_print(LOG_EMERG, "#1: Quorum Dissolved\n");
 		rg_set_inquorate();
 		member_list_update(NULL);/* Clear member list */
 		rg_lockall(L_SYS);
 		rg_doall(RG_INIT, 1, "Emergency stop of %s\n");
 #ifndef USE_OPENAIS
-		log_printf(LOG_DEBUG, "Invalidating local VF cache\n");
+		logt_print(LOG_DEBUG, "Invalidating local VF cache\n");
 		vf_invalidate();
 #endif
-		log_printf(LOG_DEBUG, "Flushing resource group cache\n");
+		logt_print(LOG_DEBUG, "Flushing resource group cache\n");
 		kill_resource_groups();
-		rg_set_uninitialized();
+		rg_clear_initialized(0);
 		return -1;
 	} else if (!rg_quorate()) {
 
 		rg_set_quorate();
 		rg_unlockall(L_SYS);
 		rg_unlockall(L_USER);
-		log_printf(LOG_NOTICE, "Quorum Regained\n");
+		logt_print(LOG_NOTICE, "Quorum Regained\n");
 	}
 
 	old_membership = member_list();
@@ -190,7 +188,7 @@ membership_update(void)
 					port);
 
 			if (quorate == 0) {
-				log_printf(LOG_DEBUG, "Node %d is not listening\n",
+				logt_print(LOG_DEBUG, "Node %d is not listening\n",
 					new_ml->cml_members[x].cn_nodeid);
 				new_ml->cml_members[x].cn_member = 0;
 				break;
@@ -224,7 +222,7 @@ membership_update(void)
 	me = memb_online(node_delta, my_id());
 	if (me) {
 		/* Should not happen */
-		log_printf(LOG_INFO, "State change: LOCAL OFFLINE\n");
+		logt_print(LOG_INFO, "State change: LOCAL OFFLINE\n");
 		if (node_delta)
 			free_member_list(node_delta);
 		node_event(1, my_id(), 0, 0);
@@ -233,7 +231,7 @@ membership_update(void)
 
 	for (x=0; node_delta && x < node_delta->cml_count; x++) {
 
-		log_printf(LOG_INFO, "State change: %s DOWN\n",
+		logt_print(LOG_INFO, "State change: %s DOWN\n",
 		       node_delta->cml_members[x].cn_name);
 		/* Don't bother evaluating anything resource groups are
 		   locked.  This is just a performance thing */
@@ -241,7 +239,7 @@ membership_update(void)
 			node_event_q(0, node_delta->cml_members[x].cn_nodeid,
 				     0, 0);
 		} else {
-			log_printf(LOG_DEBUG, "Not taking action - services"
+			logt_print(LOG_DEBUG, "Not taking action - services"
 			       " locked\n");
 		}
 	}
@@ -255,7 +253,7 @@ membership_update(void)
 
 	me = memb_online(node_delta, my_id());
 	if (me) {
-		log_printf(LOG_INFO, "State change: Local UP\n");
+		logt_print(LOG_INFO, "State change: Local UP\n");
 		node_event_q(1, my_id(), 1, 1);
 	}
 
@@ -267,7 +265,7 @@ membership_update(void)
 		if (node_delta->cml_members[x].cn_nodeid == my_id())
 			continue;
 
-		log_printf(LOG_INFO, "State change: %s UP\n",
+		logt_print(LOG_INFO, "State change: %s UP\n",
 		       node_delta->cml_members[x].cn_name);
 		node_event_q(0, node_delta->cml_members[x].cn_nodeid, 1, 1);
 	}
@@ -290,7 +288,7 @@ lock_commit_cb(char __attribute__ ((unused)) *key,
 	char lockstate;
 
 	if (datalen != 1) {
-		log_printf(LOG_WARNING, "%s: invalid data length!\n", __FUNCTION__);
+		logt_print(LOG_WARNING, "%s: invalid data length!\n", __FUNCTION__);
 		free(data);
 		return 0;
 	}
@@ -301,18 +299,18 @@ lock_commit_cb(char __attribute__ ((unused)) *key,
 	if (lockstate == 0) {
 		rg_unlockall(L_USER); /* Doing this multiple times
 					 has no effect */
-		log_printf(LOG_NOTICE, "Resource Groups Unlocked\n");
+		logt_print(LOG_NOTICE, "Resource Groups Unlocked\n");
 		return 0;
 	}
 
 	if (lockstate == 1) {
 		rg_lockall(L_USER); /* Doing this multiple times
 				       has no effect */
-		log_printf(LOG_NOTICE, "Resource Groups Locked\n");
+		logt_print(LOG_NOTICE, "Resource Groups Locked\n");
 		return 0;
 	}
 
-	log_printf(LOG_DEBUG, "Invalid lock state in callback: %d\n", lockstate);
+	logt_print(LOG_DEBUG, "Invalid lock state in callback: %d\n", lockstate);
 	return 0;
 }
 
@@ -384,7 +382,7 @@ do_lockreq(msgctx_t *ctx, int req)
 
 #ifdef OPENAIS
 	ret = ds_write("rg_lockdown", &state, 1);
-	log_printf(LOG_INFO, "FIXME: send RG_LOCK update to all!\n");
+	logt_print(LOG_INFO, "FIXME: send RG_LOCK update to all!\n");
 #else
 	ret = vf_write(m, VFF_IGN_CONN_ERRORS, "rg_lockdown", &state, 1);
 	free_member_list(m);
@@ -422,7 +420,7 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 	/* Peek-a-boo */
 	sz = msg_receive(ctx, msg_hdr, sizeof(msgbuf), 1);
 	if (sz < (int)sizeof (generic_msg_hdr)) {
-		log_printf(LOG_ERR,
+		logt_print(LOG_ERR,
 		       "#37: Error receiving header from %d sz=%d CTX %p\n",
 		       nodeid, sz, ctx);
 		goto out;
@@ -444,21 +442,21 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 	/* Decode the header */
 	swab_generic_msg_hdr(msg_hdr);
 	if ((msg_hdr->gh_magic != GENERIC_HDR_MAGIC)) {
-		log_printf(LOG_ERR,
+		logt_print(LOG_ERR,
 		       "#38: Invalid magic: Wanted 0x%08x, got 0x%08x\n",
 		       GENERIC_HDR_MAGIC, msg_hdr->gh_magic);
 		goto out;
 	}
 
 	if ((int)msg_hdr->gh_length != sz) {
-		log_printf(LOG_ERR, "#XX: Read size mismatch: %d %d\n",
+		logt_print(LOG_ERR, "#XX: Read size mismatch: %d %d\n",
 		       ret, msg_hdr->gh_length);
 		goto out;
 	}
 
 	switch (msg_hdr->gh_command) {
 	case RG_STATUS:
-		//log_printf(LOG_DEBUG, "Sending service states to CTX%p\n",ctx);
+		//logt_print(LOG_DEBUG, "Sending service states to CTX%p\n",ctx);
 		if (send_rg_states(ctx, msg_hdr->gh_arg1) == 0)
 			need_close = 0;
 		break;
@@ -484,7 +482,7 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 	case RG_ACTION_REQUEST:
 
 		if (sz < (int)sizeof(msg_sm)) {
-			log_printf(LOG_ERR,
+			logt_print(LOG_ERR,
 			       "#39: Error receiving entire request (%d/%d)\n",
 			       ret, (int)sizeof(msg_sm));
 			ret = -1;
@@ -503,7 +501,7 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 
 			if (msg_send(ctx, msg_sm, sizeof (SmMessageSt)) <
 		    	    (int)sizeof (SmMessageSt))
-				log_printf(LOG_ERR, "#40: Error replying to "
+				logt_print(LOG_ERR, "#40: Error replying to "
 				       "action request.\n");
 			ret = -1;
 			goto out;
@@ -543,7 +541,7 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 	case RG_EVENT:
 		/* Service event.  Run a dependency check */
 		if (sz < (int)sizeof(msg_sm)) {
-			log_printf(LOG_ERR,
+			logt_print(LOG_ERR,
 			       "#39: Error receiving entire request (%d/%d)\n",
 			       ret, (int)sizeof(msg_sm));
 			ret = -1;
@@ -566,7 +564,7 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		if (!member_online(msg_hdr->gh_arg1))
 			break;
 
-		log_printf(LOG_NOTICE, "Member %d shutting down\n",
+		logt_print(LOG_NOTICE, "Member %d shutting down\n",
 		       msg_hdr->gh_arg1);
 	       	member_set_state(msg_hdr->gh_arg1, 0);
 		node_event_q(0, msg_hdr->gh_arg1, 0, 1);
@@ -580,7 +578,7 @@ dispatch_msg(msgctx_t *ctx, int nodeid, int need_close)
 		break;
 
 	default:
-		log_printf(LOG_DEBUG, "unhandled message request %d\n",
+		logt_print(LOG_DEBUG, "unhandled message request %d\n",
 		       msg_hdr->gh_command);
 		break;
 	}
@@ -610,18 +608,18 @@ handle_cluster_event(msgctx_t *ctx)
 	switch(ret) {
 	case M_PORTOPENED:
 		msg_receive(ctx, NULL, 0, 0);
-		log_printf(LOG_DEBUG, "Event: Port Opened\n");
+		logt_print(LOG_DEBUG, "Event: Port Opened\n");
 		membership_update();
 		break;
 	case M_PORTCLOSED:
 		/* Might want to handle powerclosed like membership change */
 		msg_receive(ctx, NULL, 0, 0);
-		log_printf(LOG_DEBUG, "Event: Port Closed\n");
+		logt_print(LOG_DEBUG, "Event: Port Closed\n");
 		membership_update();
 		break;
 	case M_NONE:
 		msg_receive(ctx, NULL, 0, 0);
-		log_printf(LOG_DEBUG, "NULL cluster message\n");
+		logt_print(LOG_DEBUG, "NULL cluster message\n");
 		break;
 	case M_OPEN:
 		newctx = msg_new_ctx();
@@ -642,31 +640,32 @@ handle_cluster_event(msgctx_t *ctx)
 		
 	case M_OPEN_ACK:
 	case M_CLOSE:
-		log_printf(LOG_DEBUG, "I should NOT get here: %d\n",
+		logt_print(LOG_DEBUG, "I should NOT get here: %d\n",
 		       ret);
 		break;
 	case M_STATECHANGE:
 		msg_receive(ctx, NULL, 0, 0);
-		log_printf(LOG_DEBUG, "Membership Change Event\n");
+		logt_print(LOG_DEBUG, "Membership Change Event\n");
 		if (running) {
 			rg_unlockall(L_SYS);
 			membership_update();
 		}
 		break;
-	case 998:
-		break;
-	case 999:
 	case M_TRY_SHUTDOWN:
 		msg_receive(ctx, NULL, 0, 0);
-		log_printf(LOG_WARNING, "#67: Shutting down uncleanly\n");
+		logt_print(LOG_WARNING, "#67: Shutting down uncleanly\n");
 		rg_set_inquorate();
 		rg_doall(RG_INIT, 1, "Emergency stop of %s");
-		rg_set_uninitialized();
+		rg_clear_initialized(0);
 #if defined(LIBCMAN_VERSION) && LIBCMAN_VERSION >= 2
 		/* cman_replyto_shutdown() */
 #endif
 		running = 0;
 		break;
+	case M_CONFIG_UPDATE:
+		msg_receive(ctx, NULL, 0, 0);
+		need_reconfigure = 1;
+		break;
 	}
 
 	return ret;
@@ -697,7 +696,7 @@ dump_internal_state(char *loc)
 int
 event_loop(msgctx_t *localctx, msgctx_t *clusterctx)
 {
- 	int n = 0, max, ret, oldver, newver;
+ 	int n = 0, max, ret;
 	fd_set rfds;
 	msgctx_t *newctx;
 	struct timeval tv;
@@ -726,6 +725,8 @@ event_loop(msgctx_t *localctx, msgctx_t *clusterctx)
 		if (msg_fd_isset(clusterctx, &rfds)) {
 			msg_fd_clr(clusterctx, &rfds);
 			handle_cluster_event(clusterctx);
+			if (need_reconfigure)
+				break;
 			continue;
 		}
 
@@ -766,10 +767,10 @@ event_loop(msgctx_t *localctx, msgctx_t *clusterctx)
 	if (!running)
 		return 0;
 
-	if (need_reconfigure || check_config_update(&oldver, &newver)) {
+	if (need_reconfigure) {
 		need_reconfigure = 0;
 		configure_rgmanager(-1, 0);
-		config_event_q(oldver, newver);
+		config_event_q();
 		return 0;
 	}
 
@@ -849,7 +850,7 @@ configure_rgmanager(int ccsfd, int dbg)
 	if (ccs_get(ccsfd, "/cluster/rm/@central_processing", &v) == 0) {
 		set_central_events(atoi(v));
 		if (atoi(v))
-			log_printf(LOG_NOTICE,
+			logt_print(LOG_NOTICE,
 			       "Centralized Event Processing enabled\n");
 		free(v);
 	}
@@ -857,11 +858,11 @@ configure_rgmanager(int ccsfd, int dbg)
 	if (ccs_get(ccsfd, "/cluster/rm/@status_poll_interval", &v) == 0) {
 		status_poll_interval = atoi(v);
 		if (status_poll_interval >= 1) {
-			log_printf(LOG_NOTICE,
+			logt_print(LOG_NOTICE,
 			       "Status Polling Interval set to %d\n",
 			       status_poll_interval);
 		} else {
-			log_printf(LOG_WARNING, "Ignoring illegal "
+			logt_print(LOG_WARNING, "Ignoring illegal "
 			       "status_poll_interval of %s\n", v);
 			status_poll_interval = 10;
 		}
@@ -885,7 +886,7 @@ clu_initialize(cman_handle_t *ch)
 
 	*ch = cman_init(NULL);
 	if (!(*ch)) {
-		log_printf(LOG_NOTICE, "Waiting for CMAN to start\n");
+		logt_print(LOG_NOTICE, "Waiting for CMAN to start\n");
 
 		while (!(*ch = cman_init(NULL))) {
 			sleep(1);
@@ -899,12 +900,12 @@ clu_initialize(cman_handle_t *ch)
 		   and log in -- this will cause the plugin to not select any
 		   node group (if any exist).
 		 */
-		log_printf(LOG_NOTICE, "Waiting for quorum to form\n");
+		logt_print(LOG_NOTICE, "Waiting for quorum to form\n");
 
 		while (cman_is_quorate(*ch) == 0) {
 			sleep(1);
 		}
-		log_printf(LOG_NOTICE, "Quorum formed\n");
+		logt_print(LOG_NOTICE, "Quorum formed\n");
 	}
 
 }
@@ -914,14 +915,14 @@ void
 wait_for_fencing(void)
 {
         if (node_has_fencing(my_id()) && !fence_domain_joined()) {
-		log_printf(LOG_INFO, "Waiting for fence domain join operation "
+		logt_print(LOG_INFO, "Waiting for fence domain join operation "
 		       "to complete\n");
 
 		while (fence_domain_joined() == 0)
 			sleep(1);
-		log_printf(LOG_INFO, "Fence domain joined\n");
+		logt_print(LOG_INFO, "Fence domain joined\n");
 	} else {
-		log_printf(LOG_DEBUG, "Fence domain already joined "
+		logt_print(LOG_DEBUG, "Fence domain already joined "
 		       "or no fencing configured\n");
 	}
 }
@@ -982,20 +983,10 @@ main(int argc, char **argv)
 		}
 	}
 
-	/*
-	   Set up logging / foreground mode, etc.
-	 */
-#if 0
-	if (debug)
-		clu_set_loglevel(LOG_DEBUG);
-	if (foreground)
-		clu_log_console(1);
-#endif
-
 	if (!foreground && (geteuid() == 0)) {
 		daemon_init(argv[0]);
 		if (wd && !debug && !watchdog_init())
-			log_printf(LOG_NOTICE, "Failed to start watchdog\n");
+			logt_print(LOG_NOTICE, "Failed to start watchdog\n");
 	}
 
 	setup_signal(SIGINT, flag_shutdown);
@@ -1010,7 +1001,7 @@ main(int argc, char **argv)
 		unblock_signal(SIGSEGV);
 	}
 
-	init_logging();
+	init_logging(NULL, foreground, (debug? LOG_DEBUG : SYSLOGLEVEL));
 	clu_initialize(&clu);
 	if (cman_init_subsys(clu) < 0) {
 		perror("cman_init_subsys");
@@ -1032,7 +1023,7 @@ main(int argc, char **argv)
 	}
 	set_my_id(me.cn_nodeid);
 
-	log_printf(LOG_INFO, "I am node #%d\n", my_id());
+	logt_print(LOG_INFO, "I am node #%d\n", my_id());
 
 	wait_for_fencing();
 
@@ -1041,22 +1032,22 @@ main(int argc, char **argv)
 	   read the resource group trees from ccsd.
 	 */
 	configure_rgmanager(-1, debug);
-	log_printf(LOG_NOTICE, "Resource Group Manager Starting\n");
+	logt_print(LOG_NOTICE, "Resource Group Manager Starting\n");
 
 	if (init_resource_groups(0, do_init) != 0) {
-		log_printf(LOG_CRIT, "#8: Couldn't initialize services\n");
+		logt_print(LOG_CRIT, "#8: Couldn't initialize services\n");
 		return -1;
 	}
 
 	if (msg_listen(MSG_SOCKET, RGMGR_SOCK, me.cn_nodeid, &local_ctx) < 0) {
-		log_printf(LOG_CRIT,
+		logt_print(LOG_CRIT,
 		       "#10: Couldn't set up cluster message system: %s\n",
 		       strerror(errno));
 		return -1;
 	}
 
 	if (msg_listen(MSG_CLUSTER, &port, me.cn_nodeid, &cluster_ctx) < 0) {
-		log_printf(LOG_CRIT,
+		logt_print(LOG_CRIT,
 		       "#10b: Couldn't set up cluster message system: %s\n",
 		       strerror(errno));
 		return -1;
@@ -1074,14 +1065,14 @@ main(int argc, char **argv)
 	 */
 #ifdef OPENAIS
 	if (ds_init() < 0) {
-		log_printf(LOG_CRIT, "#11b: Couldn't initialize SAI AIS CKPT\n");
+		logt_print(LOG_CRIT, "#11b: Couldn't initialize SAI AIS CKPT\n");
 		return -1;
 	}
 
 	ds_key_init("rg_lockdown", 32, 10);
 #else
 	if (vf_init(me.cn_nodeid, port, NULL, NULL) != 0) {
-		log_printf(LOG_CRIT, "#11: Couldn't set up VF listen socket\n");
+		logt_print(LOG_CRIT, "#11: Couldn't set up VF listen socket\n");
 		return -1;
 	}
 
@@ -1100,14 +1091,14 @@ main(int argc, char **argv)
 			   be ignored here */
 			msg_close(local_ctx);
 			++shutdown_pending;
-			log_printf(LOG_NOTICE, "Shutting down\n");
+			logt_print(LOG_NOTICE, "Shutting down\n");
 			pthread_create(&th, NULL, shutdown_thread, NULL);
 		}
 	}
 
 	if (rg_initialized())
 		cleanup(cluster_ctx);
-	log_printf(LOG_NOTICE, "Shutdown complete, exiting\n");
+	logt_print(LOG_NOTICE, "Shutdown complete, exiting\n");
 	clu_lock_finished(rgmanager_lsname);
 	cman_finish(clu);
 	
