@@ -1,25 +1,5 @@
 #!/bin/bash
 
-#
-#  Copyright Red Hat, Inc. 2002-2004
-#  Copyright Mission Critical Linux, Inc. 2000
-#
-#  This program is free software; you can redistribute it and/or modify it
-#  under the terms of the GNU General Public License as published by the
-#  Free Software Foundation; either version 2, or (at your option) any
-#  later version.
-#
-#  This program is distributed in the hope that it will be useful, but
-#  WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#  General Public License for more details.
-#
-#  You should have received a copy of the GNU General Public License
-#  along with this program; see the file COPYING.  If not, write to the
-#  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
-#  MA 02139, USA.
-#
-
 #
 # NFS/CIFS file system mount/umount/etc. agent
 #
@@ -444,9 +424,9 @@ killMountProcesses()
                                 else
                                         kill -TERM $pid
                                 fi
-                        done < <(fuser -vm $mp | \
+			done < <(fuser -vm $mp 2>&1 | \
                             grep -v PID | \
-                            sed 's;^'$mp';;' | \
+			    sed 's;^'$mp:';;' | \
                             awk '{print $4,$2,$1}' | \
                             sort -u -k 1,3)
                 fi
